// -*-C++-*-
//
// File:      OmSourceParameters.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Aug  9 16:49:17 MEST 2002

#ifndef OmSourceParameters_h
#define OmSourceParameters_h

#include "DopeVector.h"
#include <list>



// equivalent of f90::type(OmSourceParameter)
struct OmSourceParameter {
    OmSourceParameter() :
      //    box,
    number(0), 
    xcoord(0), 
    ycoord(0),
    rightascension(0), 
    declination(0),  
    positionerror(0),
    exposure(0), 
    significance(0),
    rawcountrate(0), 
    rawcountrateerror(0),
    corrcountrate(0),
    corrcountrateerror(0),  //
    magnitude(0),
    magnitudeerror(0),  
    rawbackgroundcountrate(0),
    rawbackgroundcountrateerror(0),
    corrbackgroundcountrate(0),
    corrbackgroundcountrateerror(0),
    psf1correction(0),
    psf2correction(0),
    coincidencecorrection(0), 
    tdscorr(0),
    semia(0),
    semiaerr(0),
    semib(0),
    semiberr(0),
    theta(0),  
    thetaerr(0),
    photometryradius(0),   // was 0
    axialRatio(1.0),
    alpha(0.0), 
    innerbackgroundradius(0),
    outerbackgroundradius(0),
    nosourcepixels(0),
    extensionflag(0),
    confusionflag(0), 
    qualityflag(0),
    iMin(0), iMax(0), jMin(0), jMax(0) 

  {}   
  //    struct Box box;
    int number; 
    float xcoord; 
    float ycoord;
    double rightascension; 
    double declination;  
    float positionerror;
    float exposure; 
    float significance;
    float rawcountrate; 
    float rawcountrateerror;
    float corrcountrate;
    float corrcountrateerror;  //
    float magnitude;
    float magnitudeerror;  
    float rawbackgroundcountrate;
    float rawbackgroundcountrateerror;
    float corrbackgroundcountrate;
    float corrbackgroundcountrateerror;
    float psf1correction;
    float psf2correction;
    float coincidencecorrection; 
    float tdscorr;
    float semia, semiaerr;
    float semib, semiberr;
    float theta, thetaerr;  
    float photometryradius;
    float axialRatio;
    float alpha;   
    float innerbackgroundradius;
    float outerbackgroundradius;
    int nosourcepixels;
    Integer1 extensionflag;
    Integer1 confusionflag; 
    Integer2 qualityflag;
    Integer2 iMin; 
    Integer2 iMax; 
    Integer2 jMin; 
    Integer2 jMax; 

   };



/*****************************************************
 The observation details
*****************************************************/
struct PhotometryParameters {

  PhotometryParameters() : 
    apertureRadius(0),  
  calApertureRadius(0) 
      
  {
  }
    float apertureRadius;
    float calApertureRadius;
};
class OmSourceParameters;
F90MODPROCDEF(void,omsourceparameter_mod,new_)(OmSourceParameters*,const int&, const int&);
F90MODPROCDEF(void,omsourceparameter_mod,delete_)(OmSourceParameters*);


class OmSourceParameters : public DopeVector<OmSourceParameter>
{
public:
  // f90:: allocate(OmSourceParameter(length))
  OmSourceParameters(int length)
  {
    int lbound=1;
    int ubound=length;
    F90MODPROC(omsourceparameter_mod, new_)(this, lbound, ubound);
  }

  // f90:: allocate(OmSourceParameter(lbound:ubound))
  OmSourceParameters(int lbound,int ubound)
  {
    F90MODPROC(omsourceparameter_mod,new_)(this,lbound,ubound);
  }
  
  // f90:: deallocate(OmSourceParameter)
  ~OmSourceParameters()
  {
    F90MODPROC(omsourceparameter_mod,delete_)(this);
  }
private:
   // disabled member functions
  OmSourceParameters();
  OmSourceParameters(const OmSourceParameters&);
  OmSourceParameters& operator=(const OmSourceParameters&);
};



struct LsqData {
  LsqData() : noStars(0), backGround(0.0), sigmaX(0.0), sigmaY(0.0) {}; 
  
  Integer1 noStars;
  bool toFit[10];  
  float backGround;  
  float sigmaX;
  float sigmaY;
  float xPos[10];
  float yPos[10];
  float i0[10];
  int exitValue;
};





#endif
