/*************************************************************************
*
*
*
* Created 2008 July 25th by CB
*
*
*************************************************************************/

#include "omqualitymap.h"
#include "qualitypixels.h"
#include <iostream>
#include <vector>       
#include "Dal.h"         // c++ interface to c fits input/output
#include "sourcelist.h"  // For input/output to/from FITS files
#include "utility.h"     // 
#include "qfile.h"       // QT header file for file operations


/**************************************************
*
* The following file contains function prototypes
*
**************************************************/
#include "omqualitymapdefs.h"


/*********************************************************
*
* This is the entry point to the main progam
*
*
********************************************************/
void omqualitymap()
{

    string operatingMode = stringParameter( "mode" );

    /***************************************************************
    * If this is true then we are updating the QUALITY image with
    * the quality data from the sources in the srclist file (case 1).
    * Otherwise, we are updating the quality flag of each source
    * with the information stored in the QUALITY image of the
    * set file (case 2)
    * 
    * case 1 - omquality map should be run after omdetect and before
    * omatt
    *
    * case 2 - omquality map should be run after running omdetect
    *          on a mosaiced image
    *
    ****************************************************************/
    string inSet = stringParameter( "set" );            // The name of the image file
    checkFileExists( inSet );                           // Check to see that the file exits

    string outSet = stringParameter( "outset");         // The output file  


    string srcListSet = stringParameter( "srclistset"); // The name of the source-list file
    checkFileExists( srcListSet );                      // Check to see that the file exits

     
    if (QFile::exists( outSet ) )                       // Check to see if the output file exists
    {
        
        errstr << "The file " << outSet << " will be overwritten" << Sas::warning("overWriteFile");

  
    }
   
    if (operatingMode == "setqualityimage" || operatingMode == "SETQUALITYIMAGE" ) // Update the quality map image
    { 

        if( srcListSet == outSet)
	{
	    errstr << "Source-list" << srcListSet << " equals output image file" << Sas::fatal("badFile");

        }

        if(inSet != outSet)
	dataSetServer -> copy( inSet, outSet );      // Copy the input image file to the specified output file


        /******************************************************************************
        *
        * Modify the quality image with quality-flag information from the source-list
        *
        *****************************************************************************/ ;

        errstr << "Updating the QUALITY image in file " << inSet << Sas::message(Msg::App, Msg::Verbose);
 
        setQualityImagePixels(inSet, outSet);


    }
    else 
    { 
        if(srcListSet != outSet)
	    dataSetServer -> copy( srcListSet, outSet ); // Updating the source-list file     


        /********************************************************
        *
        * Set the quality-array pixels
        *
        **********************************************************/
        errstr << "Updating the source-list quality flags in file " << srcListSet << Sas::message(Msg::App, Msg::Verbose);
 
        updateTheQualityList(inSet, outSet, srcListSet);

    }

}  // End of the main program
/***********************************************************
*
*
* Sets the source quality flags using information from
* the quality image
*
*
***********************************************************/
void updateTheQualityList(string inSet, string outSet, string srcListSet)
{


    /****************************************************************************
    *
    * Load various columns of data from the SRCLIST table in the source-list file
    *
    *****************************************************************************/


    DataSet *dataSet = dataSetServer->open(srcListSet, Dal::Read);
    if( !dataSet)
    {

      errstr << "Could not open file" << srcListSet << Sas::fatal("badFile");

    }
    
    if ( !dataSet->hasBlock( "SRCLIST") ) 
    {
       
        dataSetServer->close(dataSet); // Finished with the file  
	errstr << "No SRCLIST table in file" << srcListSet << Sas::fatal("badFile");

    }
    const Table *table = dataSet->table("SRCLIST");
  
    int size = table->rows();

    if(size == 0)
    {
     
        dataSetServer->close(dataSet); // Finished with the file  
	errstr << "The SRCLIST table in file " << srcListSet << " has zero rows- exiting" << Sas::warning("EmptySRCLISTtable");

        return;

    }

   
    /********************************************************
    *
    * Load the source-list quality-flags
    * 
    *
    ******************************************************/
    vector<int16> qualityList;

     
    if ( !LoadVectorInt16ColumnData(table, "QFLAG", size, qualityList) )
    {
      
        dataSetServer->close(dataSet); // Finished with the file
               
	errstr << "Could not load column QFLAG from table SRCLIST in file " << srcListSet << Sas::fatal("badFile");


    }
   
        


    /*************************************************************************
    *
    * Load the RA
    *
    ***************************************************************************/ 
       
    vector<double> raList;
    if ( !LoadVectorDoubleColumnData(table, "RA", size, raList) )
    {
     
        dataSetServer->close(dataSet); // Finished with the file
                    
	errstr << "Could not load column RA from table SRCLIST in file " 
               << srcListSet << Sas::fatal("badFile");

    }

    /********************************************************
    *
    * Load the Dec column
    *
    ******************************************************/
    vector<double> decList;
    if ( !LoadVectorDoubleColumnData(table, "DEC", size, decList) )
    {
       
        dataSetServer->close(dataSet); // Finished with the file
               
	errstr << "Could not load column DEC from table SRCLIST in file " 
               << srcListSet << Sas::fatal("badFile");


    }

    dataSetServer->close(dataSet); // Finished with the file

    /**************************************************************
    *
    * Modify the quality-list
    *
    **************************************************************/    
    modifyQualityList(inSet, outSet, raList, decList, qualityList);


}
/****************************************************************************
*
* inset should be an unrotated mod-8 corrected image file
*
* 
* We are setting quality-flag pixel bits on the un-rotated 16-bit quality image
* using both the output source-list from omdetect and the actual image
* 
*
*
****************************************************************************/
void setQualityImagePixels(string inSet, string outSet)
{ 

  //    cout << "setqualityimagepixels line 277 inset=" << inSet << " outset=" << outSet << endl;

    errstr << "Modifying the quality array in file " << outSet 
	   << Sas::message(Msg::App, Msg::Sparse);

    /*******************************************************************
    *
    * Load various source parameters from the SWS source-list file
    * struct OmSourceParameter contains fields for most of the parameters
    *
    ********************************************************************/
    vector<OmSourceParameter> parameter;

    string srcListSet = stringParameter("srclistset");
    int size = 0;
    string filter;
    if( !loadSourceParameters( srcListSet, "SRCLIST", size, parameter, filter) )
    {
         errstr << "Could not load data from table SRCLIST in file " << srcListSet << Sas::fatal("badFile");
    }
  
    /***********************************************************************
    * 
    * Only quality bit 0 (bad pixel) will initially be set (by omcosflag)
    *
    * Load the PRIMARY image from the input file
    *
    ***********************************************************************/  
    int nx = 0, ny = 0; // The x and y image dimensions
    vector<vector<float> >  primaryImage = loadFloatImage(inSet, "PRIMARY", nx, ny);

    /******************************************************************
    *
    * Load the QUALITY image from the input file
    *
    ***********************************************************************/  
    vector<vector<int16> >  qualityImage = loadInt16Image(inSet, "QUALITY", nx, ny);

 
    /***********************************************************************************
    *
    * Set the QUALITY image pixels in the centre and corner regions
    *
    * Need to obtain some window parameters from the input file for converting
    * image x/y coordinates to detector x/y coordinates
    *
    ************************************************************************************/
    DataSet *dataSet = dataSetServer->open( outSet, Dal::Modify );

    if( !dataSet)
    {      
	errstr << "Could not open file " << outSet << Sas::fatal("badFile");

    }    


    /******************************************************************
    *
    * Need the following to convert coordinates to full-frame coordinates
    *
    ******************************************************************/
    int x0 = dataSet->attribute("WINDOWX0)")->asInt();      
    int y0 = dataSet->attribute("WINDOWY0)")->asInt();          
 
  
    int binX = (int)pow(2.0, dataSet->attribute("BINAX1")->asInt() );  
 
    if ( dataSet->attribute("BINBPE")->asBool() )
    { 
        binX *= 2;
 
    }

    /***********************************************************************
    *
    * Set the quality-flag image pixels using the source-list and the image
    *
    ************************************************************************/
    setQualityImagePixels(primaryImage, qualityImage, parameter, x0, y0, binX);



 

    /******************************************************************
    *
    * Delete the old QUALITY image and replace it with the new one
    *
    ******************************************************************/
    dataSet->deleteBlock("QUALITY");    

  

    /******************************************************************
    *
    * Add the new QUALITY image
    *
    ******************************************************************/  
    addInt16Image( dataSet, "QUALITY", qualityImage);




    dataSetServer->close(dataSet); // Finished with the file


}
/**************************************************************
*
* Checks that the file "file" exists
*
**************************************************************/
void checkFileExists( string file )
{
    if (!QFile::exists( file ) )
    {
             
        errstr << "File " << file << " does not exist" << Sas::fatal("badFile");


    }



};

/****************************************************************************
*
*  Modifies the quality flags of each source using the quality image
* 
*
*
****************************************************************************/
void modifyQualityList(string inSet, string outSet, 
             vector<double> &raList, vector<double> &decList, 
             vector<int16> &qualityList)
{ 


    if(raList.size() == 0) return;
  

    //  cout << "line 397 modifyqualitylist inset=" << inSet << " outset=" << outSet << endl;
    //  cout << "ralist.size=" << raList.size() << endl;
    /***************************************************************
    *
    *
    * Load the quality image
    *
    *
    ****************************************************************/  
    int nx = 0, ny = 0;
    vector<vector<int16> >  qualityImage = loadInt16Image(inSet, "QUALITY", nx, ny);

    int noFlags = F90MODPROC( flags_mod, getnoqualityflags)(); // The number of quality flags
    




    double cdelt1 = 0.0;
    double cdelt2 = 0.0;  // Dec plate-scale (degrees)
    double crpix1 = 0.0;
    double crpix2 = 0.0;
    double crval1 = 0.0;
    double crval2 = 0.0;
    
    DataSet *dataSet = dataSetServer->open( inSet, Dal::Read);
    
  
    if ( dataSet->hasAttribute("CDELT1") )
    {
     

   
        cdelt1 = dataSet->attribute("CDELT1")->asReal();  // RA plate-scale (degrees and negative)
        cdelt2 = dataSet->attribute("CDELT2")->asReal();  // Dec plate-scale (degrees)
        crpix1 = dataSet->attribute("CRPIX1")->asReal();
        crpix2 = dataSet->attribute("CRPIX2")->asReal();
        crval1 = dataSet->attribute("CRVAL1")->asReal();
        crval2 = dataSet->attribute("CRVAL2")->asReal();
    
    }
    else
    {
  


        const Block *block = dataSet->block(0);
        if ( !block->hasAttribute( "CDELT1" ) )
	{


	      errstr << "No attribute CDELT1 in file  " << outSet << "!- terminating program\n" 
	     << Sas::message(Msg::App, Msg::Sparse);

  

       

             dataSetServer->close(dataSet); // Finished with the file
                    
	     errstr<< "Attribute CDELT1 does not exist in file " << outSet << Sas::fatal("badFile");



        }

        cdelt1 = block->attribute("CDELT1")->asReal();  // RA plate-scale (degrees and negative)
        cdelt2 = block->attribute("CDELT2")->asReal();  // Dec plate-scale (degrees)
        crpix1 = block->attribute("CRPIX1")->asReal();
        crpix2 = block->attribute("CRPIX2")->asReal();
        crval1 = block->attribute("CRVAL1")->asReal();
        crval2 = block->attribute("CRVAL2")->asReal();
    }    
  
    
    dataSetServer->close(dataSet); // Finished with the file

    /****************************************************************
    *
    * Modify the quality flag of each source using the quality image
    *
    ****************************************************************/
  


    for(unsigned i = 0; i < qualityList.size(); i++) // Loop through the sources
    {
     
        /**************************************************************
	*
	* Convert from celestial to tangent-plane coordinates
        *
        ***************************************************************/

        double dx = 0.0;
        double dy = 0.0;
 

   
        F90MODPROC( convert_mod, convert2)(&raList[i], &decList[i], &crval1, &crval2, &dx, &dy);
    
             
        double x = dx / cdelt1 + crpix1;
        double y = dy / cdelt2 + crpix2;
 


        int j = int(x - 0.9999);
        j = min(j, nx - 1); 
        j = max(0, j);
        int k = int(y - 0.9999);
        k = max(0, k);
        k = min(k, ny - 1);
       
        /****************************************************************
	*
	* Check the pixels around the source to see if any have a quality
        * flag set- if so set the source quality flag
        *
        *****************************************************************/
        checkPixels(nx, ny, j, k, noFlags, qualityImage, qualityList[i] );

  

    }


    /********************************************************
    *
    * Replace the QFLAG column
    *
    ********************************************************/
    if ( !addColumn( outSet, "SRCLIST",  "QFLAG", "", "", (int)qualityList.size(), qualityList) )
    {
      
        errstr << "Could not modify column QFLAG in file " << outSet << " ..."
	       << Sas::message(Msg::App, Msg::Sparse);
    }


}

/***********************************************************
*
* Checks the pixels around a source and sets the quality flag
* if any pixels have a quality flag set
*
*
***********************************************************/
void checkPixels(int nx, int ny, int j, int k, int noFlags, 
		 vector<vector<int16> > &qualityImage, int16 &source)
{

    int dx = 1; 
    int il = max(0, j - dx);
    int iu = min(nx - 1, j + dx);
    int jl = max(0, k - dx);
    int ju = min(ny - 1, k + dx);

    for(int l = 0; l < noFlags; l++)
    {
       
        if ( bitSet( source, l) ) continue;	

        for (int i1 = il; i1 <= iu; i1++)
	{
	    for(int j1 = jl; j1 <= ju; j1++)
	    {
	        if( qualityImage[i1][j1] < 0) continue;
	        if ( bitSet( qualityImage[i1][j1], l)) 
		{
	            setBit(source, l);	
                }             
            }
	    
        }
          
       
    }

}
/************************************************************
*
*  Given image x, y coordinates converts them
*  to full-frame unbinned coordinates  
*
*
************************************************************/
void ConvertCoordsToFullFrame(float xCoord, float yCoord, 
                              int x0, int y0, int xBin, int yBin, 
                              float & xOut, float &yOut)
{
    
    xOut = xCoord * float(xBin) + float(x0);
    yOut = yCoord * float(yBin) + float(y0);


}
/********************************************************************
* Set the read-out streak flags
*
********************************************************************/
void setReadOutStreaks( const int binX, vector<vector<int16> > & qualityImage)
{


    vector<vector<int16> > image = qualityImage; 

    int nx = (int)image.size() - 1;
    int ny = (int)image[0].size() - 1;

    int bitNo = 1; // For read-out streak

   
    int dk = 4 / binX;

    for(int i = 0; i <= nx; i++)
    {
        for(int j = 0; j <= ny; j++)
	{
   
            if ( bitSet( qualityImage[i][j], bitNo))
	    {

	       int l1 = max(0, i - dk);
               int l2 = min(nx, i + dk);
 
                for(int k = 0; k <= ny; k++)
		{
		    for (int l = l1; l <= l2; l++) 
		        setBit( image[l][k], bitNo);
	

                }
	        break;
            }  
 

	} 

    }

    /**************************************************************
    *
    * Replace quality image with the new one
    *
    **************************************************************/

    qualityImage = image;



}
/*********************************************************
*
*
* Sets the quality-flag bits on the QUALITY image using
* both the source-list and the PRIMARY image
*
*
*
*
* QFLAG source quality flags convention
* BIT NO             FLAGGING                         Integer
*
*   0                BAD PIXEL                                 1
*   1                READ-OUT STREAK                           2
*   2                SMOKE-RING                                4
*   3                SOURCE ON STAR-SPIKE                      8
*   4                Mod-8 source                             16
*   5                Source within central enhancement        32
*   6                Source lies near to a bright source      64
*   7                Near an edge                            128
*   8                point-source within extended source     256
*   9                Weird source                            512
*
*
* CFLAG source confusion flags convention
* BIT NO             FLAGGING                                                Integer
*
*   0                one or more source within 0.5-1.0 aperture radius       1
*   1                one or more source within 0.5 aperture radius           2
******************************************************************************/
void setQualityImagePixels( vector<vector<float> > &image,
                            vector<vector<int16> > &qualityMap, 
                            vector<OmSourceParameter> &source,
			    int x0, int y0, int binX)
{
    

    int16 nx = (int16)image.size();
    int16 ny = (int16)image[0].size();



    /***********************************************************
    *
    * Some of the flagging is done in FORTRAN
    * To avoid code repetition using the existing Fortran routines
    * using a Fortran interface
    *
    ***********************************************************/



    F90MODPROC( addflags_mod, createqualitymap)(&nx, &ny);

   
    for(int16 i = 0; i < nx; i++)
    {
        for(int16 j = 0; j < ny; j++)
        {

	    /*******************************************************
	    * Initialise pixel i, j of the fortran quality map
            *
            * NB the only quality-flag set here will be BIT 0 
            ******************************************************/
	    F90MODPROC(addflags_mod, setqualitypixel)(&i, &j, &qualityMap[i][j]);

        }
    }
 
    int size = (int)source.size();

    OmSourceParameter sourceList[size]; // For Fortran use
    
    for(int i = 0; i < size; i++)
        sourceList[i] = source[i];


    F90MODPROC(addflags_mod, addsourceflags)( &size, sourceList, &x0, &y0, &binX);



    /******************************************************************************
    *
    * Get the modified quality image
    *
    *******************************************************************************/
    for(int16 i = 0; i < nx; i++)
    {
        for(int16 j = 0; j < ny; j++)
        {
	    qualityMap[i][j] = F90MODPROC(addflags_mod, getqualitypixel)(&i, &j);

        }
    }

    /************************************************************
    *
    * Finished with the Fortran quality map
    *
    *************************************************************/
    F90MODPROC( addflags_mod, deletequalitymap)();

    /********************************************************************
    *
    * Set the central-region pixels (BIT 5)
    *
    ********************************************************************/
    setCentralRegionPixels( x0, y0, binX, qualityMap );


    /********************************************************************
    *
    * Set the image edge pixels (BIT 7)
    *
    ********************************************************************/
    setEdgePixels( binX, qualityMap);


    /********************************************************************
    *
    * Set the corner-region pixels (BIT 7)
    *
    ********************************************************************/
    setCornerPixels( x0, y0, binX, qualityMap);

}
    
