#include "qualitypixels.h"
#include "Dal.h"
#include <vector>
#include "utility.h"
#include <cmath>






/****************************************************************************
* QFLAG source quality flags convention
* BIT NO             FLAGGING                         Integer
*
*   0                BAD PIXEL                                 1
*   1                READ-OUT STREAK                           2
*   2                SMOKE-RING                                4
*   3                SOURCE ON STAR-SPIKE                      8
*   4                Mod-8 source                             16
*   5                Source within central enhancement        32
*   6                Source lies near to a bright source      64
*   7                Near an edge                            128 
*   8                point-source within extended source     256 
*   9                Weird source                            512
********************************************************************************/




/**************************************************************
*
*  For bad pixel
*
**************************************************************/
void SetQualityPixels0::setPixels( double x, double y, int16 qualityFlag, 
                          vector<vector<int16> >  &image)
{

                 
      
      if ( bitSet( qualityFlag, bitNo)) 
      {
	  int nx = (int)image.size();
          int ny = (int)image[0].size();

               
          int j = int(x - 0.9999);
          j = min(j, nx - 1); 
          j = max(0, j);
          int k = int(y - 0.9999);
          k = max(0, k);
          k = min(k, ny - 1);
  
 


          int dx = 12 / binAxis; 
          float maxRadius = float(10 / binAxis);

          int il = max(0, j - dx);
          int iu = min(nx - 1, j + dx);
          int jl = max(0, k - dx);
          int ju = min(ny - 1, k + dx);
          for (int i1 = il; i1 <= iu; i1++)
	  {
	      float dx = float(i1 + 1) - x;      
	      for(int j1 = jl; j1 <= ju; j1++)
	      {
                  float dy = float(j1 + 1) - y;      
	          float radius = hypot(dx, dy);
                  if(radius > maxRadius) continue;  
                  if( image[i1][j1] < 0) continue;
	    
	              
		  setBit(image[i1][j1], bitNo);	            
                       
                  
	      }
          }
      }
  


};





/**************************************************************
*
* For read-out streak
*
**************************************************************/


void SetQualityPixels1::setPixels( double x, double y, int16 qualityFlag, 
                          vector<vector<int16> >  &image)
{

                 
      
      if ( bitSet( qualityFlag, bitNo)) 
      {
	  int nx = (int)image.size();
          int ny = (int)image[0].size();

               
          int j = int(x - 0.9999);
          j = min(j, nx - 1); 
          j = max(0, j);
          int k = int(y - 0.9999);
          k = max(0, k);
          k = min(k, ny - 1);
  
 


          int dx = 10; 
          int il = max(0, j - dx);
          int iu = min(nx - 1, j + dx);
          int jl = max(0, k - dx);
          int ju = min(ny - 1, k + dx);
          for (int i1 = il; i1 <= iu; i1++)
	  {
                    
	      for(int j1 = jl; j1 <= ju; j1++)
	      {  
                  if( image[i1][j1] < 0) continue;
	    
	              
		  setBit(image[i1][j1], bitNo);	            
                       
                  
	      }
          }
      }
   


};



/**************************************************************
*
* For smoke-ring
*
**************************************************************/
void SetQualityPixels2::setPixels( double x, double y, int16 qualityFlag, 
                          vector<vector<int16> >  &image)
{

                 
      
      if ( bitSet( qualityFlag, bitNo)) 
      {
	  int nx = (int)image.size();
          int ny = (int)image[0].size();

               
          int j = int(x - 0.9999);
          j = min(j, nx - 1); 
          j = max(0, j);
          int k = int(y - 0.9999);
          k = max(0, k);
          k = min(k, ny - 1);
  
 


          int dx = 10; 
          int il = max(0, j - dx);
          int iu = min(nx - 1, j + dx);
          int jl = max(0, k - dx);
          int ju = min(ny - 1, k + dx);
          for (int i1 = il; i1 <= iu; i1++)
	  {
                    
	      for(int j1 = jl; j1 <= ju; j1++)
	      {  
                  if( image[i1][j1] < 0) continue;
	    
	              
		  setBit(image[i1][j1], bitNo);	            
                       
                  
	      }
          }
      }
   


};


/**************************************************************
*
* For diffraction spike
*
**************************************************************/

void SetQualityPixels3::setPixels( double x, double y, int16 qualityFlag, 
                          vector<vector<int16> >  &image)
{

                 
      
      if ( bitSet( qualityFlag, bitNo)) 
      {
	  int nx = (int)image.size();
          int ny = (int)image[0].size();

               
          int j = int(x - 0.9999);
          j = min(j, nx - 1); 
          j = max(0, j);
          int k = int(y - 0.9999);
          k = max(0, k);
          k = min(k, ny - 1);
  
 


          int dx = 10; 
          int il = max(0, j - dx);
          int iu = min(nx - 1, j + dx);
          int jl = max(0, k - dx);
          int ju = min(ny - 1, k + dx);
          for (int i1 = il; i1 <= iu; i1++)
	  {
                    
	      for(int j1 = jl; j1 <= ju; j1++)
	      {  
                  if( image[i1][j1] < 0) continue;
	    
	              
		  setBit(image[i1][j1], bitNo);	            
                       
                  
	      }
          }
      }
   


};


/**************************************************************
*
* For mod-8 pattern
*
**************************************************************/


void SetQualityPixels4::setPixels( double x, double y, int16 qualityFlag, 
                          vector<vector<int16> >  &image)
{

                 
      
      if ( bitSet( qualityFlag, bitNo)) 
      {
	  int nx = (int)image.size();
          int ny = (int)image[0].size();

               
          int j = int(x - 0.9999);
          j = min(j, nx - 1); 
          j = max(0, j);
          int k = int(y - 0.9999);
          k = max(0, k);
          k = min(k, ny - 1);
  
 


          int dx = 10; 
          int il = max(0, j - dx);
          int iu = min(nx - 1, j + dx);
          int jl = max(0, k - dx);
          int ju = min(ny - 1, k + dx);
          for (int i1 = il; i1 <= iu; i1++)
	  {
                    
	      for(int j1 = jl; j1 <= ju; j1++)
	      {  
                  if( image[i1][j1] < 0) continue;
	    
	              
		  setBit(image[i1][j1], bitNo);	            
                       
                  
	      }
          }
      }
   


};


/**************************************************************
*
* For inside central enhancement
*
**************************************************************/
void SetQualityPixels5::setPixels( double x, double y, int16 qualityFlag, 
                          vector<vector<int16> >  &image)
{

                 
      
      if ( bitSet( qualityFlag, bitNo)) 
      {
	  int nx = (int)image.size();
          int ny = (int)image[0].size();

               
          int j = int(x - 0.9999);
          j = min(j, nx - 1); 
          j = max(0, j);
          int k = int(y - 0.9999);
          k = max(0, k);
          k = min(k, ny - 1);
  
 


          int dx = 10; 
          int il = max(0, j - dx);
          int iu = min(nx - 1, j + dx);
          int jl = max(0, k - dx);
          int ju = min(ny - 1, k + dx);
          for (int i1 = il; i1 <= iu; i1++)
	  {
                    
	      for(int j1 = jl; j1 <= ju; j1++)
	      {  
                  if( image[i1][j1] < 0) continue;
	    
	              
		  setBit(image[i1][j1], bitNo);	            
                       
                  
	      }
          }
      }
   


};



/********************************************************************
* Set the read-out streak flags
*
********************************************************************/
void dilateImage( vector<vector<int16> > & qualityImage, int bitNumber, int dk)
{


    vector<vector<int16> > image = qualityImage; 

    int nx = (int)image.size() - 1;
    int ny = (int)image[0].size() - 1;

  

    for(int i = 0; i <= nx; i++)
    {

	int i1 = max( 0, i - dk);
        int i2 = min(nx, i + dk);
  
        for(int j = 0; j <= ny; j++)
	{
   
	    if ( !bitSet( qualityImage[i][j], bitNumber)) continue; 
	    {

	       int j1 = max(0, j - dk);
               int j2 = min(ny, j + dk);
               for(int k = i1; k <= i2; k++)
	       {
                   for(int l = j1; l <= j2; l++)
	           {
                        
		       setBit( image[k][l], bitNumber);
                   }
                   

               }
             
            }  
  


	} 

    }

    qualityImage = image;



}

 




