
#include "Dal.h"
#include <iostream>
#include "sourcelist.h"
#include <qdir.h>
#include <qstring.h>
#include <typeinfo>
#include <vector>
#include "SasDalUtils.h"
#include "dal_utilities.h"
#include "DataSubspace.h"
#include <cstdlib>
#include "OmSourceParameters.h"



using namespace SasDalUtils;  


string fits = "FTZ";
void setFits( string fit)
{
  fits = fit;

}

/********************************************************
 *
 *
 ******************************************************/
int GetNoRows(const string& fileName, const string& tableName) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


    if ( !dataSet->hasBlock( tableName) ) 
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
 
        return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    int size = table->rows();
  
    dataSetServer->close(dataSet); // Finished with the file
 
    return size;
}

/********************************************************
 *
 *
 ******************************************************/
float GetAttribute(const string& fileName, const string& tableName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return 0;    


    if (!dataSet->hasBlock(tableName))    
    {
       cout << "Table doesn't exist\n";
       dataSetServer->close(dataSet); // Finished with the file
 
       return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    float value = -1.0;
    if (table->hasAttribute(name))
        value = table->attribute(name)->asReal();


    dataSetServer->close(dataSet); // Finished with the file
  
    return value;    

}


/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const string & value) 
{
    // Open the file

  //  cout << "addattribute line 87\n";
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( !dataSet )
        return;    

    //    cout << "Line 94 adding attribute " << name << " =" << value << endl;

    *dataSet->addAttribute( name, comment ) = value;


    dataSetServer->close(dataSet); // Finished with the file
  
}

/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const bool & value) 
{
    // Open the file
  


    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( ! dataSet )
      return;    

  
    *dataSet->addAttribute( name, comment ) = value;




    dataSetServer->close(dataSet); // Finished with the file
  
    

}
/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const float & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}


/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const double & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}


/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const int & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}








/********************************************************
 *
 *
 ******************************************************/
bool GetBooleanAttribute(const string& fileName, const string& tableName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return 0;    


   if (!dataSet->hasBlock(tableName)) 
   
   {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    bool value = false;
    if (table->hasAttribute(name))
        value = table->attribute(name)->asBool();


    dataSetServer->close(dataSet); // Finished with the file
  
    return value;    

}


/********************************************************
 *
 *
 ******************************************************/
string GetAttribute(const string& fileName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) 
    {
      cout << "Bug - could not open file " << fileName << endl;
      exit(0);

    }    


  
   string value = " ";
   if (dataSet->hasAttribute(name))
     value = (dataSet->attribute(name)->asString()).c_str();



   dataSetServer->close(dataSet); // Finished with the file
  
   return value;    

}
/********************************************************
 *
 *
 ******************************************************/
vector<float> GetAttributes(const string& fileName, const string& tableName, const string name[], int n) 
{

    vector<float> value;
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return value;    


   if (!dataSet->hasBlock(tableName)) 
   {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return value;

    }

   const Table *table = dataSet->table(tableName);
 
   for (int i = 0; i < n; i++)
   { 

       if (table->hasAttribute(name[i]))
	 {
	   float f = table->attribute(name[i])->asReal();
      
           value.push_back(f);
	 }
       else
   value.push_back(-1.0);
	
   }
   dataSetServer->close(dataSet); // Finished with the file
 
  
   return value;    

}

/********************************************************
*
*
******************************************************/
float *LoadColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
    dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    float *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
        data = new float[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};



/********************************************************
*
*
******************************************************/
float *LoadColumnData(const Table *table,  const string& columnName)
{
     
   
  
    int size = table->rows();
   
  
    float *data = 0;

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	
     


      
        float *xData = columnData->real32Data();


    



        data = new float[size];


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return data;

};

  
  


/********************************************************
*
*
******************************************************/
double *LoadDoubleColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
    dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i = 0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};


/********************************************************
*
*
******************************************************/
double *LoadDoubleColumnData(const Table *table, const string& columnName)
{
     
   
      
    int size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
 
    return data;
};







/********************************************************
*
*
******************************************************/


double *LoadColumnData1(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
          
    dataSetServer->close(dataSet); // Finished with the file
   
  
    return data;
};

/********************************************************
*
*
******************************************************/

Integer1 *LoadInt1ColumnData(const string& fileName, 
                             const string& tableName, 
                             const string& columnName, 
                             int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);


    if (!dataSet)
   
      {
	cout << "failure to open data set\n";
      return 0;    
      }

         // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return 0;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    Integer1 *data = 0;
    
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();


       if (!columnData) exit(0);
	
   
      
       //        int *xData = columnData->int32Data();
       //    Integer1 *xData = columnData->int8Data();
        int8 *xData = columnData->int8Data();

        data = new Integer1[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = (Integer1)xData[i]; 
       
	}
     }
     
       
    dataSetServer->close(dataSet); // Finished with the file
      

  
    return data;
};
/********************************************************
*
*
******************************************************/

Integer1 *LoadInt1ColumnData(const Table *table, const string& columnName)
{
   
    
  
    int size = table->rows();
   
  
    Integer1 *data = 0;
    
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();


       if (!columnData) exit(0);
	
   
      
       //        int *xData = columnData->int32Data();
       //    Integer1 *xData = columnData->int8Data();
        int8 *xData = columnData->int8Data();

        data = new Integer1[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = (Integer1)xData[i]; 
       
	}
     }
     
       
 
      

  
    return data;
};

/********************************************************
*
*
******************************************************/

Integer2 *LoadInt2ColumnData(const Table *table, const string& columnName)
{
   
    
  
    int size = table->rows();
   
  
    Integer2 *data = 0;
    
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();


       if (!columnData) exit(0);
	
   
      
        int16 *xData = columnData->int16Data();

        data = new Integer2[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = (Integer2)xData[i]; 
       
	}
     }
     
       
 
      

  
    return data;
};

/********************************************************
*
*
******************************************************/

int *LoadIntColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);


    if (!dataSet)
   
      {
	cout << "failure to open data set\n";
      return 0;    
      }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return 0;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    int *data = 0;
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int *xData = columnData->int32Data();
        data = new int[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
       
	}
     }
     
      
    dataSetServer->close(dataSet); // Finished with the file
 

  
    return data;
};










/********************************************************
*
*
******************************************************/

vector<string> LoadVectorStringColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<string> data(0);
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName))     
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
      
        return data;

    }


    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
     
        
  

	char * d = columnData -> stringData();
        int width = columnData -> parent() -> elements(); 
        char text[ width + 1]; 
        for(unsigned long i = 0; i < columnData -> count(); i++, d += columnData -> parent() -> elements( ))
	{

          strcpy(text, " ");
	  strncpy(text, d,  width );
          text[width] = '\0'; 
	  //	  cout << i << ") " << text << endl;
	
          data.push_back( string(text) );

      
    	}
        delete columnData;




     }
     
    dataSetServer->close(dataSet); // Finished with the file

     
    return data;
};


/********************************************************
*
*
******************************************************/

vector<int> LoadVectorIntColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<int> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int *xData = columnData->int32Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
  dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};




/********************************************************
*
*
******************************************************/

vector<int8> LoadVectorInt8ColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<int8> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int8 *xData = columnData->int8Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
  dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};








/********************************************************
*
*
******************************************************/

int *LoadIntColumnData(const Table *table, const string& columnName)
{
   

    int size = table->rows();
   
  
    int *data = 0;
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int *xData = columnData->int32Data();
        data = new int[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
       
	}
     }
     
       
        

  
    return data;
}

/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter, char c)
{
  
    QDir d;
    

    d.setFilter (QDir::Files);
    //    d.setSorting(QDir::Size | QDir::Name); // Sort by name
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
  




        QString s = fi->fileName().latin1();

        if (s[0] == c && s.contains(filter) && s.contains( fits ) && !s.contains("new") )
	{


  
            string filter = GetAttribute( s, "FILTER" );


 
            if (filter != "GRISM1" && filter != "GRISM2" && filter != "BLOCKED") 
	    {


	        fileList.push_back(s);
             
            }
        }
        ++it;
    } 
    return fileList;

}

/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter, char* filter1, char c)
{
  
    QDir d;
    

    d.setFilter (QDir::Files );
    //    d.setSorting(QDir::Size | QDir::Name); // Sort by name
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
  


        QString s = fi->fileName().latin1();

        if (s[0] == c && ( s.contains(filter) || s.contains(filter1) )  && s.contains( fits ) && !s.contains("new") )
	{


  
            string filter = GetAttribute( s, "FILTER" );


 
            if (filter != "GRISM1" && filter != "GRISM2" && filter != "BLOCKED") 
	    {


	        fileList.push_back(s);
           
            }
        }
        ++it;
    } 
    return fileList;

}


/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter)
{
  
    QDir d;
    d.setFilter (QDir::Files );
  
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
        QString s = fi->fileName().latin1();
    
        if (s.contains(filter))
	{
	    fileList.push_back(s);
            
        }
        ++it;
    } 
 
    return fileList;

}

/***************************************************************
*
****************************************************************/
bool deleteRowFromTable(const string& fileName,
                        const string& tableName,
                        const int rowNumber)
{


    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);


    if (!dataSet)
    {
	cout << "failure to open data set\n";
        return false;    
    }


    if (!dataSet->hasBlock(tableName)) 
   
    {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return false;

    }

    Table *table = dataSet->table(tableName);
  
    table->deleteRows(rowNumber, 1);


    dataSetServer->close(dataSet); // Finished with the file
 
    return true;



}



/******************************************************
*
******************************************************/



void setColumnData( const int size, const ColumnData *columnData, const int8* data, const int iStart)
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}



/******************************************************
*
******************************************************/



void setColumnData( const int size, const ColumnData *columnData, const vector<int8> &data, const int iStart )
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}


/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const int16* data, const int iStart )
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}
/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int16> &data, const int iStart)
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}
/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const int32* data, const int iStart )
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int32> & data, const int iStart )
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}

/******************************************************
*
******************************************************/


void setColumnData( const int size, const ColumnData *columnData, const real32* data, const int iStart )
{
    real32 * r32 = columnData->real32Data();
    for(int i = 0; i < size; i++)
        r32[iStart + i] = data[i];

}
/******************************************************
*
******************************************************/


void setColumnData( const int size, const ColumnData *columnData, const vector<real32>& data, const int iStart )
{
    real32 * r32 = columnData->real32Data();
    for(int i = 0; i < size; i++)
        r32[iStart + i] = data[i];

}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const real64* data, const int iStart )
{
    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<real64> &data, const int iStart )
{
    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const char* data, const int iStart)
{

  if (!columnData) return;
  //    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
      cout << data[iStart + i] << endl;


}
/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
               vector<real32> &data)
{
     




    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   

   
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }
    // Get the size of the table
    vector<unsigned long> size(2);
    size[0] = 1;
    size[1] = nData;
  


    Table *table = dataSet->table(tableName, Dal::Modify);
    Column *column = 0;
    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

  
      
 
        // Create the column
        column = table->addColumn( columnName, Column::Real32, comment, units , size);
    }
 
    ColumnData *columnData = column->data();


    setColumnData( ( int)size[1], columnData, data);




    delete columnData;

    dataSetServer->close(dataSet); // Finished with the file



 



          
    return true;
};

/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
               vector<int16> &data)
{
     

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
  

 
    if (!dataSet->hasBlock(tableName)) 
    {

        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }




    // Get the size of the table
    vector<unsigned long> size(2);
    size[0] = 1;
    size[1] = nData;
  
 
    Table *table = dataSet->table(tableName, Dal::Modify);
    Column *column = 0;
    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {      
 
        // Create the column
        column = table->addColumn( columnName, Column::Int16, comment, units , size);
    }
 
    ColumnData *columnData = column->data();


    setColumnData( ( int)size[1], columnData, data);

 
    delete columnData;

    dataSetServer->close(dataSet); // Finished with the file

          
    return true;
};


/********************************************************
*
*
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real32> &data)
{
       
    // Get the size of the table
    vector<unsigned long> size(2);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real32, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;
          
    return true;

};







/******************************************************
*
******************************************************/
bool appendToColumn( Table *table, 
                     const char* columnName, 
                     int iStart,   
		     int nData,
                     vector<real32> data)
{
     

  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );

    delete columnData;

    return true;
};


/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real64> &data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(2);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real64, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( (int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/****************************************************************
 *
 ****************************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName, 
		     int iStart,    
		     int nData,
                     vector<real64> data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart);


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int8> &data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(2);
    size[0] = 1;

    size[1] = nData;
  

   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int8, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( (int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/**********************************************************************************
*
**********************************************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName, 
		     int iStart,
		     int nData,
                     vector<int8> data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );

    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
using namespace SasDalUtils;  

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int16> &data)
{
     

    // Get the size of the table
    vector<unsigned long> size(2);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int16, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( (int)size[1], columnData, data);


    delete columnData;


          
    return true;
};
/********************************************************************
*
*
********************************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName, 
		     int iStart,    
		     int nData,
                     vector<int16> data)
{
     

  

  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );


    delete columnData;


          
    return true;
};




/********************************************************
*
*
******************************************************/
using namespace SasDalUtils;  

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int32> &data)
{
     






  
    // Get the size of the table
    vector<unsigned long> size(2);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int32, comment, units , size);
 
    ColumnData *columnData = column->data();

 
    setColumnData( (int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/***************************************************
*
*
**************************************************/
bool appendToColumn(Table *table, 
                     const char* columnName,                  
		     int iStart,   
		     int nData,
                     vector<int32> data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data, iStart );


    delete columnData;


          
    return true;
};




/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName, 
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,
               int columnWidth,   
	       int nData,
               vector<string>& data)
{




 
     

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }





    // Read the table
    Table *table = dataSet->table(tableName, Dal::Modify);




  
    // Get the size of the table
    vector<unsigned long> size(1);

    size[0] = columnWidth;

   

    // Create the column
    Column *column = 0;
    if ( table->hasColumn( columnName )) 
    {
        column = table->column( columnName );


    }

    else
    {
  
        column = table->addColumn( columnName, Dal::DString, comment, units , size);
    
    }



    if (column )
    { 
        ColumnData *columnData = column->data();

        if (!columnData) exit(0);
	
	for (int i = 0; i < nData; i++) 
        {

	  if(data[i].length() > 1) {  
	    cout << i <<  ") sourcelist.cc line 1378 adding comment " << data[i] << " to source-list " << fileName 
                 << " comment length=" << data[i].length() << endl;
	    setStringColumnRow( columnData, i, data[i] );
	    cout << "Line 1980\n";
 
	  } 
	}
        delete columnData;

     }
    else
        exit(0);     
         
    dataSetServer->close(dataSet); // Finished with the file
 
  
    return true;
};







//  vector<unsigned long> strlen;
//  strlen.push_back(10);
//  col = tab->addColumn("string",Column::DString,"string data","",strlen);


//  d = col->data();
//  char *s = d->stringData();
//  int len = d->size()[0];
//  assert( len == 10 );
//  for (unsigned long i = 0; i < nRows; ++i) {
//    char	ss[ len + 1 ];
//    sprintf( ss, "string%ld", i );
    
//    strncpy(s + d->position(i), ss, len);
//  }
//  delete d;









/********************************************************
*
*
******************************************************/
bool appendToColumn(const string fileName, 
                     const char* tableName, 
                     const char* columnName, 
                     int columnWidth,
		     int iStart,    
		     int nData,
                     vector<string> data)
{
     

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }


    // Read the table
    Table *table = dataSet->table(tableName);
  
    // Get the size of the table
    vector<unsigned long> size(1);

    size[0] = columnWidth;

   
    // Create the column
    Column *column = table->column( columnName );
 

    if (column )
    { 
        ColumnData *columnData = table->column(columnName)->data();

        if (!columnData) exit(0);
	
        using namespace SasDalUtils;  


	for (int i = 0; i < nData; i++) 
        {

	  setStringColumnRow( columnData, iStart + i, data[i] ); 
      
	}
        delete columnData;

     }
    else
      exit(0);     
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return true;
};








/**********************************************************************
*  Load all the source data
*
**********************************************************************/
bool loadSourceParameters(const string &fileName, const char* tableName,
			  int& size, vector < OmSourceParameter >& parameters, string & filter) 
                     

{
  

  // cout << "Loadsourceparameters line 2199\n";


    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {

      // cout << "Loadsourceparameters line 2209\n";



        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }

    // cout << "Loadsourceparameters line 2219\n";



    filter = (dataSet->attribute( "FILTER" )->asString()).c_str();
    float exposure =  (dataSet->attribute( "EXPOSURE" ))->asReal();
 
    // cout << "Loadsourceparameters line 218\n";


    // Open the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
 

    parameters.empty();
    parameters.resize( size ); 

    
    // cout << "Loadsourceparameters line 2232\n";



    // Load the columns one by one

    

    // ************ XPOS *************************

    if (table->hasColumn( "XPOS" )) 
    { 
        const ColumnData *columnData = table->column( "XPOS" )->data();
	    


       
        real32 *xData = columnData->real32Data();
       

    
	for (int i = 0; i < size; i++) 
        {
	  parameters[i].xcoord = xData[i];           
          parameters[i].exposure = exposure; 
          parameters[i].number = i + 1;        
	}
     


     }
     
  
    // cout << "Loadsourceparameters line 2265\n";



    // ************ YPOS *************************
    if (table->hasColumn( "YPOS" )) 
    { 
        const ColumnData *columnData = table->column( "YPOS" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].ycoord = xData[i]; 
       
	}
     }
     
    // ************ POSERR *************************



    if (table->hasColumn( "POSERR" )) 
    { 
        const ColumnData *columnData = table->column( "POSERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].positionerror = xData[i]; 
       
	}
     }
     
    // ************ RA *************************


    // cout << "Loadsourceparameters line 2313\n";

    if (table->hasColumn( "RA" )) 
    { 
        const ColumnData *columnData = table->column( "RA" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rightascension = xData[i]; 
       
	}
     }
     

   
    // ************ DEC *************************

    if (table->hasColumn( "DEC" )) 
    { 
        const ColumnData *columnData = table->column( "DEC" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].declination = xData[i]; 
       
	}
     }
     


  


    //cout << "Loadsourceparameters line 2360\n";



    if (table->hasColumn( "RATE" )) 
    { 
        const ColumnData *columnData = table->column( "RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawcountrate = xData[i]; 
       
	}
     }
     

    // cout << "Loadsourceparameters line 2383\n";




    if (table->hasColumn( "RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawcountrateerror = xData[i]; 
       
	}
     }


    // cout << "Loadsourceparameters line 2407\n";




    if (table->hasColumn( "BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawbackgroundcountrate = xData[i]; 
       
	}
     }
     
     
    // cout << "Loadsourceparameters line 2431\n";




    if (table->hasColumn( "SIGNIFICANCE" )) 
    { 
        const ColumnData *columnData = table->column( "SIGNIFICANCE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].significance = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "CORR_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrate = xData[i]; 
       
	}
     }
     

    //cout << "Loadsourceparameters line 2476\n";




    if (table->hasColumn( "CORR_RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrateerror = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "CORR_BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrbackgroundcountrate = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "PSF1_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF1_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf1correction = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "PSF2_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF2_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf2correction = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "MAG" )) 
    { 
        const ColumnData *columnData = table->column( "MAG" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitude = xData[i]; 
       
	}
     }
     
    if (table->hasColumn( "MAGERR" )) 
    { 
        const ColumnData *columnData = table->column( "MAGERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitudeerror = xData[i]; 
       
	}
     }
     

    
    if (table->hasColumn( "NOPIXELS" )) 
    { 
        const ColumnData *columnData = table->column( "NOPIXELS" )->data();
	    

	
       
        int32 *xData = columnData->int32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].nosourcepixels = xData[i]; 
       	 
       

	}
     }
     


   
 
     
    if (table->hasColumn( "APERTURE" )) 
    { 
        const ColumnData *columnData = table->column( "APERTURE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].photometryradius = xData[i]; 
       
	}
     }





    if (table->hasColumn( "FWHM_MAJ" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ" )->data();
	    

     

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semia = xData[i]; 
       
	}
    }

    if (table->hasColumn( "FWHM_MAJ_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ_ERR" )->data();
	    

   

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiaerr = xData[i]; 
       
	}
    }





    if (table->hasColumn( "FWHM_MIN" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semib = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "FWHM_MIN_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiberr = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "PA" )) 
    { 
        const ColumnData *columnData = table->column( "PA" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].theta = xData[i]; 
       
	}
     }
     

    if (table->hasColumn( "PA_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "PA_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].thetaerr = xData[i]; 
	         

	}
     }
     

  


    if (table->hasColumn( "EFLAG" )) 
    { 
        const ColumnData *columnData = table->column( "EFLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].extensionflag = xData[i]; 
	  
        
	}
     }
     
 
   
    if (table->hasColumn( "QFLAG" )) 
    { 
        const ColumnData *data = table->column( "QFLAG" )->data();
	    
 
      
	int type = data->dataType();
	//        cout << "datatype=" << type << endl;
        switch (type ) 
        {
	    case 2:
	    {     
	        int8 *xData = data->int8Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	           
       
	        }
                break;
	    }

	    case 4: 
	    {     
	        int16 *xData = data->int16Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }
	    default:
	    {
	        cout << "bug";
	        exit(0);
            } 

        }




     }
     



    if (table->hasColumn( "CFLAG" )) 
    { 
        const ColumnData *columnData = table->column( "CFLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].confusionflag = xData[i]; 
       
	}
     }
            
    dataSetServer->close(dataSet); // Finished with the file



      
    return true;
} 

/**********************************************************************
*  Load all the source data from an OBS file
*
**********************************************************************/
bool loadSourceParameters(const string &fileName, const char* tableName,
			  int& size, vector < OmSourceParameter >& parameters, bool &oldData) 
                     

{
  
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }


    oldData = false;



    // Open the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
 

    parameters.empty();
    parameters.resize( size ); 

    



    // Load the columns one by one

    







    // ************ XPOS *************************

    if (table->hasColumn( "XPOS" )) 
    { 
        const ColumnData *columnData = table->column( "XPOS" )->data();
	    


       
        real32 *xData = columnData->real32Data();
       

    
	for (int i = 0; i < size; i++) 
        {
	  parameters[i].xcoord = xData[i];           
          parameters[i].number = i + 1;        

        
	}
     


     }
     
  
    // ************ YPOS *************************
    if (table->hasColumn( "YPOS" )) 
    { 
        const ColumnData *columnData = table->column( "YPOS" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].ycoord = xData[i]; 
       
	}
     }










    if (table->hasColumn( "POSERR" )) 
    { 
        const ColumnData *columnData = table->column( "POSERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].positionerror = xData[i]; 
       
	}
        delete columnData;




     }
     
    // ************ RA *************************



    if (table->hasColumn( "RA" )) 
    { 
        const ColumnData *columnData = table->column( "RA" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rightascension = xData[i]; 
       
	}

        delete columnData;




     }
     

   
    // ************ DEC *************************

    if (table->hasColumn( "DEC" )) 
    { 
        const ColumnData *columnData = table->column( "DEC" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].declination = xData[i]; 
       
	}

        delete columnData;



     }
     







    // ************ significance *************************
    if (table->hasColumn( "SIGNIFICANCE" )) 
    { 
        const ColumnData *columnData = table->column( "SIGNIFICANCE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].significance = xData[i]; 
       
	}
        delete columnData;


     }
     else
     {

        

          vector<string> name;
          name .push_back( "UVW2_SIGNIF");
          name .push_back( "UVM2_SIGNIF");
          name .push_back( "UVW1_SIGNIF");
          vector< vector<float> > signif;
          int k = 0;
      

      

          for(unsigned j = 0; j < name.size(); j++)
	  {

              if (table->hasColumn( name[j] )) 
              { 
                  const ColumnData *columnData = table->column( name[j] )->data();
                  real32 *xData = columnData->real32Data();
                  k++;
	      
	       

                  signif.resize(k);
                  signif[k - 1].resize(size);
	          for (int i = 0; i < size; i++) 
                  {
	              signif[k - 1][i] = xData[i]; 
       
	          }

                  delete columnData;
	      }

	  }
          if( k > 0)
	  {
            
              for(int i = 0; i < size; i++)
	      {
               
	      
                  float maxSig = 0.0; 
                  for(int j = 0; j < k; j++)
	          {
		      maxSig = max( maxSig, signif[j][i] );  
                  }
              
                  parameters[i].significance = maxSig;
              } 
	  }

	 

     }







  

    string columnName = "MEAN_MAJ_AXIS";
    if ( !table->hasColumn( columnName ))
    {
        columnName = "FWHM_MAJ";
     
        if ( !table->hasColumn( columnName ) )
	  columnName = "";
        
 
    }



    if ( columnName != "") 
    { 
        const ColumnData *columnData = table->column( columnName )->data();
	    

     

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semia = xData[i]; 
       
	}


        delete columnData;

    }






    columnName = "MEAN_MIN_AXIS";
    if ( !table->hasColumn( columnName ))
    {
        columnName = "FWHM_MIN";
     
        if ( !table->hasColumn( columnName ) )
	  columnName = "";
 
    }




    if ( columnName != "") 
    { 
        const ColumnData *columnData = table->column( columnName )->data();
	    


        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semib = xData[i]; 
       
	}
        delete columnData;

     }
     



    columnName = "MEAN_POS_ANGLE";
    if ( !table->hasColumn( columnName ))
    {
        columnName = "PA";
     
        if ( !table->hasColumn( columnName ) )
	  columnName = "";
 
    }





    if ( columnName != "") 
    { 
        const ColumnData *columnData = table->column( columnName )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].theta = xData[i]; 
       
	}

        delete columnData;


     }
     

  
    columnName = "EFLAG";



    if (table->hasColumn( "EFLAG_MAX" ))
        columnName = "EFLAG_MAX";
    
    else if (table->hasColumn( "EFLAG" ))
        columnName = "EFLAG";
    else
        columnName = "";

     
    if(columnName != "")
    { 
        const ColumnData *columnData = table->column( columnName )->data();
	    

        	
       
        int8 *xData = columnData->int8Data();
       
	for (int i = 0; i < size; i++) 
        {
	    parameters[i].extensionflag = xData[i]; 
	   
        
	}
     }
     
 
    columnName = "QFLAG";
    if (table->hasColumn( "QFLAG_COMB" ))
        columnName = "QFLAG_COMB";
    else if (table->hasColumn( "QFLAG" ))
        columnName = "QFLAG";
    else
      columnName = "";



    if ( columnName != "" ) 
    { 
        const ColumnData *data = table->column( columnName )->data();
	    

	int type = data->dataType();
	//        cout << "datatype=" << type << endl;
        switch (type ) 
        {
	    case 2:
	    {    
	        oldData = true; 
	        int8 *xData = data->int8Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }

	    case 4: 
	    {     

	    
	        int16 *xData = data->int16Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }
	    default:
	    {
	        cout << "bug";
	        exit(0);
            } 

        }

        delete data;






     }
     


    string Name = "CFLAG";
    if (table->hasColumn( "CFLAG_COMB" ))
        columnName = "CFLAG_COMB";
    else if (table->hasColumn( "CFLAG" ))
        columnName = "CFLAG";
    else
      columnName = "";



    if (columnName != "" ) 
    { 
        const ColumnData *columnData = table->column( columnName )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].confusionflag = xData[i]; 
       
	}
        delete columnData;

     }
            




    dataSetServer->close(dataSet); // Finished with the file



      
    return true;
} 
/**********************************************************************
*
*********************************************************************/
bool upDateSrcListFile(const string &fileName, const char* tableName,
		       vector< OmSourceParameter > & parameters)
{ 
                     


    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }

    // Open the table
    Table *table = dataSet->table(tableName);


    int size = table->rows();


    ColumnData *columnData = table->column( "QFLAG")->data();

	
    int16* data1 = columnData->int16Data();


    for (int i = 0; i < size; i++) 
    {
        data1[i] = parameters[i].qualityflag; 
       
    }
    columnData->release();    



    columnData = table->column( "EFLAG")->data();

	
    int8 *data = columnData->int8Data();


    for (int i = 0; i < size; i++) 
    {
        data[i] = parameters[i].extensionflag; 
       
    }
    columnData->release();    



    columnData = table->column( "CFLAG")->data();

	
    data = columnData->int8Data();


    for (int i = 0; i < size; i++) 
    {
        data[i] = parameters[i].confusionflag; 
       
    }
    columnData->release();    



    dataSetServer->close(dataSet); // Finished with the file

  

    return true;
}


/**********************************************************************
*
*
*********************************************************************/
bool loadImage( const string & imageFile, const char *name, int &nx, int &ny, float image [])
{

    
    DataSet* set = dataSetServer->open( imageFile, Dal::Read);
    if ( !set) return false;

    Array* array = set->array( name );

    ArrayData* arraydata = array->data();
    real32* theData = arraydata->real32Data();

    

    longVec arraySize = array->size();
  
    nx = arraySize[1];
    ny = arraySize[0];




    for (int i = 0; i < nx * ny; i++)
        image[i] = theData[i];
    



    dataSetServer->close(set);


 

    return true;



}
/**********************************************************************
*
*
*********************************************************************/
bool loadImage( const string & imageFile, const char *name, int &nx, int &ny, int8 image[])
{

    
    DataSet* set = dataSetServer->open( imageFile, Dal::Read);
    if ( !set) return false;

    Array* array = set->array( name );

    ArrayData* arraydata = array->data();
    int8* theData = arraydata->int8Data();

    

    longVec arraySize = array->size();
  
    nx = arraySize[1];
    ny = arraySize[0];




    for (int i = 0; i < nx * ny; i++)
        image[i] = theData[i];
    



    dataSetServer->close(set);


 

    return true;



}
/**********************************************************************
*
*
*********************************************************************/
bool loadImage( const string & imageFile, const char *name, int &nx, int &ny, bool image[])
{

    
    DataSet* set = dataSetServer->open( imageFile, Dal::Read);
    if ( !set) return false;

    Array* array = set->array( name );

    ArrayData* arraydata = array->data();
    int8* theData = arraydata->int8Data();

    

    longVec arraySize = array->size();
  
    nx = arraySize[1];
    ny = arraySize[0];

    cout << "Line 2263 nx=" << nx << " ny=" << ny << endl;
    for (int i = 0; i < nx * ny; i++)
    {
      if ( theData[i] > 0) 
	image[i] = true;
      else
        image[i] = false;
    
    }

    dataSetServer->close(set);
 
    return true;


}


/**********************************************************************
*
*********************************************************************/
bool createSrcListFile(const string newSrcFile, const string oldSrcFile, vector< OmSourceParameter > & parameters)
{ 
                     
    dataSetServer->copy( oldSrcFile, newSrcFile );

    // Open the file
    DataSet *dataSet = dataSetServer->open(newSrcFile, Dal::Modify);
  
    if ( !dataSet ) return false;
 

    dataSet->deleteBlock( "SRCLIST" );

    // Add the table
    Table *table = dataSet->addTable( "SRCLIST", parameters.size() );


    vector<real32> data;
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].xcoord );


    if ( !addColumn( table, "XPOS", "pixels", "x coordinate",   
			   (int)parameters.size(), data) )
      return false;



    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].ycoord );

    if ( !addColumn( table, "YPOS", "pixels", "y coordinate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();


    vector<real64> data1;
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].rightascension );


    if ( !addColumn( table, "RA", "degrees", "Right-ascension",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].declination );


    if ( !addColumn( table, "DEC", "degrees", "Declination",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();


    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].positionerror );


    if ( !addColumn( table, "POSERR", "arc secs", "Position error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /**********************************************************************
     * Count rates
     ********************************************************************/

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrate );


    if ( !addColumn( table, "RATE", "counts/sec", "Raw count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrateerror );


    if ( !addColumn( table, "RATE_ERR", "counts/sec", "Raw count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawbackgroundcountrate );


    if ( !addColumn( table, "BACKGROUND_RATE", "counts/sec", "Raw background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].significance );


    if ( !addColumn( table, "SIGNIFICANCE", "", "Signa-to-noise",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrate );


    if ( !addColumn( table, "CORR_RATE", "counts/sec", "Corrected count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrateerror );


    if ( !addColumn( table, "CORR_RATE_ERR", "counts/sec", "Corrected count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrbackgroundcountrate );


    if ( !addColumn( table, "CORR_BACKGROUND_RATE", "counts/sec", "Corrected background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    /*****************************************************************************
     * Corrections applied
     ****************************************************************************/ 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf1correction );


    if ( !addColumn( table, "PSF1_CORR", "", "PSF1 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf2correction );


    if ( !addColumn( table, "PSF2_CORR", "", "UV PSF2 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /*************************************************************************************
     * Instrumental magnitudes
     ************************************************************************************/
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitude);


    if ( !addColumn( table, "MAG", "mag", "Instrumental magnitude",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitudeerror);


    if ( !addColumn( table, "MAGERR", "mag", "Instrumental magnitude error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    vector<int> data3;
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( parameters[i].nosourcepixels);


    if ( !addColumn( table, "NOPIXELS", "", "No of source pixels (extended sources)",   
			   (int)parameters.size(), data3) )
      return false;


    data3.clear();    

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].photometryradius);


    if ( !addColumn( table, "APERTURE", "unbinned pixels", "Extraction aperture radius (point-sources)",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semia);


    if ( !addColumn( table, "FWHM_MAJ", "arcsec", "FWHM major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiaerr);


    if ( !addColumn( table, "FWHM_MAJ_ERR", "arcsecs", "FWHM major-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semib);


    if ( !addColumn( table, "FWHM_MIN", "arcsecs", "FWHM minor-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiberr);


    if ( !addColumn( table, "FWHM_MIN_ERR", "arcsecs", "FWHM minor-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].theta);


    if ( !addColumn( table, "PA", "counts/sec", "Position angle of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].thetaerr);


    if ( !addColumn( table, "PA_ERR", "counts/sec", "Error in position-angel of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 




    vector<int8> data4;
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].extensionflag);


    if ( !addColumn( table, "EFLAG", "", "Source extension flag (0-point, 1-extended)",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].qualityflag);


    if ( !addColumn( table, "QFLAG", "", "Quality flag",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].confusionflag);


    if ( !addColumn( table, "CFLAG", "", "Confusion flag",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    //    table->insertRows( (int)parameters.size(), 100);

    dataSetServer->close( dataSet );

   
    return true;
}

/**********************************************************************
*  Appends new rows to the end of the source-list file
*
*********************************************************************/
bool appendToSrcListFile( const string newSrcFile, vector< OmSourceParameter > & parameters )
{ 
             


    // Open the file
    DataSet *dataSet = dataSetServer->open( newSrcFile, Dal::Modify);
  
   
    if ( !dataSet ) return false;
  

    // Update the table
    Table *table = dataSet->table( "SRCLIST", Dal::Modify );
    int iStart = table->rows();


    // Insert new rows at the end of the file
    table->insertRows( iStart, parameters.size() );
 

    vector<int32> data3;
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( iStart + i + 1 );


    if ( !appendToColumn( table, "SRCNUM",  iStart, (int)parameters.size(), data3) )
      return false;

    data3.clear();

  


    vector<real32> data;
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].xcoord );


    if ( !appendToColumn( table, "XPOS",  iStart, (int)parameters.size(), data) )
      return false;



    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].ycoord );

    if ( !appendToColumn( table, "YPOS", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();


        vector<real64> data1;
    //    for (unsigned i = 0; i < parameters.size(); i++)
    //        data1.push_back( parameters[i].rightascension );


    //    if ( !appendToColumn( table, "RA", iStart, (int)parameters.size(), data1) )
    //      return false;

    //    data1.clear();
    //    for (unsigned i = 0; i < parameters.size(); i++)
    //        data1.push_back( parameters[i].declination );


    //    if ( !appendToColumn( table, "DEC", iStart, (int)parameters.size(), data1) )
    //     return false;

	//    data1.clear();


    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].positionerror );


    if ( !appendToColumn( table, "POSERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /**********************************************************************
     * Count rates
     ********************************************************************/

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrate );


    if ( !appendToColumn( table, "RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrateerror );


    if ( !appendToColumn( table, "RATE_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawbackgroundcountrate );


    if ( !appendToColumn( table, "BACKGROUND_RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].significance );


    if ( !appendToColumn( table, "SIGNIFICANCE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrate );


    if ( !appendToColumn( table, "CORR_RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrateerror );


    if ( !appendToColumn( table, "CORR_RATE_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrbackgroundcountrate );


    if ( !appendToColumn( table, "CORR_BACKGROUND_RATE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    /*****************************************************************************
     * Corrections applied
     ****************************************************************************/ 





    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf1correction );


    if ( !appendToColumn( table, "PSF1_CORR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf2correction );


    if ( !appendToColumn( table, "PSF2_CORR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();





    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rightascension );


    if ( !appendToColumn( table, "SPB_COILOSS", iStart, (int)parameters.size(), data) )
        return false;

    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].declination );


    if ( !appendToColumn( table, "BK_COILOSS", iStart, (int)parameters.size(), data) )
        return false;

    data.clear();








 

    /*************************************************************************************
     * Instrumental magnitudes
     ************************************************************************************/
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitude);


    if ( !appendToColumn( table, "MAG", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitudeerror);


    if ( !appendToColumn( table, "MAGERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

   
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( parameters[i].nosourcepixels);


    if ( !appendToColumn( table, "NOPIXELS", iStart, (int)parameters.size(), data3) )
      return false;


    data3.clear();    

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].photometryradius);


    if ( !appendToColumn( table, "APERTURE", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semia);


    if ( !appendToColumn( table, "FWHM_MAJ", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiaerr);


    if ( !appendToColumn( table, "FWHM_MAJ_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semib);


    if ( !appendToColumn( table, "FWHM_MIN", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiberr);


    if ( !appendToColumn( table, "FWHM_MIN_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].theta);


    if ( !appendToColumn( table, "PA", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 

 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].thetaerr);


    if ( !appendToColumn( table, "PA_ERR", iStart, (int)parameters.size(), data) )
      return false;


    data.clear();
 




    vector<int8> data4;
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].extensionflag);


    if ( !appendToColumn( table, "EFLAG", iStart, (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].qualityflag);


    if ( !appendToColumn( table, "QFLAG", iStart, (int)parameters.size(), data4) )
      return false;


    data4.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].confusionflag);


    if ( !appendToColumn( table, "CFLAG", iStart, (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    dataSetServer->close( dataSet );

   
    return true;


}

/********************************************************
*
*
******************************************************/
vector<double> LoadVectorDoubleColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     

    vector<double> data;

    size = 0;
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return data;    


    if (!dataSet->hasBlock(tableName)) 
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
        return data; 
      

    }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
  
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};



/********************************************************
*
*
******************************************************/
vector<float> LoadVectorFloatColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size, int il1, int iu1)
{
     


  
   
    // Open the file
    size = 0;
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet)
    {
      exit(0);

    }    


    if (!dataSet->hasBlock(tableName)) 
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
        exit(0);  
      

    }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
    int il, iu;
    if (iu1 < il1)
    {
      il = 0;
      iu = size - 1; 
    }
    else
    {
        il = il1,
	iu = iu1;
        size = iu1 - il1 + 1;

    } 


    vector<float> data(size, -1.0);

  
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
       

        
	for (int i = il, k = 0; i <= iu; i++, k++) 
        {
	  data[k] = xData[i]; 
	 
	}

 

     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};











/********************************************************
*
*
******************************************************/
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<float> &data)
{
       
    size = table->rows();
   
 
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;
};


/********************************************************
*
*
******************************************************/
bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<double> &data)
{
       
    size = table->rows();
   
  
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
  
 
    return true;
};











/********************************************************
*
*
******************************************************/
bool LoadVectorInt16ColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<int16> &data)
{
       
    size = table->rows();
   
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        int16 *xData = columnData->int16Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;
};


/**************************************************
*
*
**************************************************/
vector<QString> loadDirectoryFiles(const string directory, 
         const int type, const string filter )
{

  //    cout << "loaddirectoryfiles line 4510 directory=" << directory << endl;  

    QDir d( directory  );
    
    d.setFilter( type ); 
    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it( *list );
    QFileInfo *fi;

    vector<QString> fileList;


    //    cout << "Loaddirectoryfiles line 4521\n";


    if (filter == "*")
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();	
            if(s.length() > 1)
	        fileList.push_back( s );
                     
    
            ++it;
        }
    }
    else 
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();
	    //	    cout << s << endl; 	
            if(s.length() > 1 && s.contains(filter) && s.contains("FTZ")  )
	        fileList.push_back( s );
                     
            ++it;
        }
    } 

    return fileList;
}
