
#ifndef UTILITY_H
#define UTILITY_H

#include "Dal.h"
#include <cmath>
#include<vector>
#include<iostream>
#include <cstdlib>
#include <cerrno>
#include <string>
#include <qmessagebox.h>

const double radiansToDegrees = (double)180.0 / M_PI;
const double radiansToArcSecs = (double)(180 * 3600) / M_PI;


const double degreesToRadians = M_PI / (double)180.0;



double distance(double r1, double d1, double r2, double d2);

double Error(double mag1, double mag2);



vector<vector<int8> >   loadInt8Image(const DataSet *set, const char *name, int &nx, int &ny);
vector<vector<int16> >  loadInt16Image(const DataSet *set, const char *name, int &nx, int &ny);


vector<vector<int8> >   loadInt8Image(const string set, const char *name, int &nx, int &ny);
vector<vector<int16> >   loadInt16Image(const string set, const char *name, int &nx, int &ny);

vector<vector<float> >   loadFloatImage(const string set, const char *name, int &nx, int &ny);




bool modifyInt16Image(DataSet *set, const char * name, const vector<vector<int16> > &image);

void addInt16Image(DataSet *set, const char * name, const vector<vector<int16> > &image);



bool isNull(int8 value);
bool isNull(int16 value);
bool isNull(int32 value);
bool isNull( float value);
bool isNull( double value);

bool isNull( char * value);






template<class kind>
void setBit(kind & value, int no)
{

    int l = sizeof(kind) * 8;
  
 
    value |= (1 << (no % l));


}
template<class kind>
void unsetBit(kind &value, int no)
{
    int l = sizeof(kind) * 8;
 
   
    value &= ~(1 << (no % l));


}
template<class kind>
bool bitSet(kind value, int no)
{

   int l = sizeof(value) * 8;
 

  return value& (1 << (no % l));

}










/***************************************************************
*
*
****************************************************************/
float Median(vector<float> &data);

 
/***************************************************************
*
*
****************************************************************/
float Mean(float *data, long nData);

/***************************************************************
*
*
****************************************************************/
float Mean(vector<float> &data);

/***************************************************************
*
*
****************************************************************/
float Median(float *data, long nData);

/***************************************************************
*
*
****************************************************************/
float Percentile(vector<float> &data, float perCent);
/***************************************************************

*
****************************************************************/
float Percentile(float *data, long size, float perCent);





/********************************************************************
*
*********************************************************************/
template<class kind>
kind minVal(vector<kind> &data)
{
    kind minData = data[0];
    for(unsigned i = 0; i < data.size(); i++)
      minData = min(minData, data[i]);

    return minData;



}

/********************************************************************
*
*********************************************************************/
template<class kind>
kind maxVal(vector<kind> &data)
{
    kind maxData = data[0];
    for(unsigned i = 0; i < data.size(); i++)
      maxData = min(maxData, data[i]);

    return maxData;



}





/**************************************************************
*
**************************************************************/
template<class kind>
bool getValue(string text, kind &value, kind xMin = 0, kind xMax = 0)
{

    char *endPtr;
    




 
    kind value1 = (kind)strtod( text.c_str(), &endPtr);
    
    if (endPtr)
    {

        if( strlen(endPtr) > 0)
        {
     
   

            for(unsigned i = 0; i < strlen(endPtr) - 2; i++)
	    {
                if( endPtr[i] != ' ') {
                    char text1[100];
                    sprintf(text1, "Invalid number - %s - Please try again!", text.c_str());
      

                    QMessageBox::critical( 0, "OMCATALOGUE", text1);

                    return false;
                }
            }

        } 
    }

    if(errno != 0) {
    
        char text1[100];
        sprintf(text1, "Invalid number - %s (%s) - Please try again!", text.c_str(), strerror(errno));
      

        QMessageBox::critical( 0, "OMCATALOGUE", text1);
 

        return false;

    }

    if(xMin != xMax)
    {
        if(value1 < xMin || value1 > xMax) {

            char text1[100];
            sprintf(text1, "Number %e outside allowed range- %e - %e - Please try again!", (double)value1, (double)xMin, (double)xMax );
      

            QMessageBox::critical( 0, "OMCATALOGUE", text1);
 

            return false;
        }





    }
    value = (kind)value1;
 
  
    return true;

   
}
#endif
